@@ -1,4 +1,3 @@
-#[allow(dead_code)]
 #[derive(thiserror::Error, Debug, strum_macros::Display)]
 pub enum GameLiftErrorType {
     ServiceCallFailed,
@@ -8,4 +7,5 @@ pub enum GameLiftErrorType {
     TerminationTimeNotSet,
     BadRequest,
     InternalServiceError,
+    UnexpectedWebSocketMessage,
 }
